; Rusted rel 0.05.2
; Created by Graechan
; Comments directed to http://gimpchat.com or http://gimpscripts.com
;
; License: GPLv3
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;    GNU General Public License for more details.
;
;    To view a copy of the GNU General Public License
;    visit: http://www.gnu.org/licenses/gpl.html
;
;
; ------------
;| Change Log |
; ------------ 
; Rel 0.01 - Initial Release
; Rel 0.02 - Added the Alpha script and increased the rust Severity
; Rel 0.03 - Added 'Rust Severity' setting to Menu
; Rel 0,04 - Bugfix releases
; Rel 0.05.1 -  Added 'Active Gradient' to background choices
; Rel 0.05.2 - Improved the output Image and added a 'Rust Coverage' setting
(define (script-fu-rusted-alpha image layer
                               pattern
							   rust-coverage
							   severity
							   metal-brightness
							   bev-width
							   conserve
							   )
	
    (gimp-image-undo-group-start image)
		

 (let* (
            (layer-width (car (gimp-drawable-width layer)))
			(layer-height (car (gimp-drawable-height layer)))
			(width (car (gimp-image-width image)))
	        (height (car (gimp-image-height image)))
			(final-width width)
			(final-height height)
			(x (car (gimp-drawable-offsets layer))) ;x 
			(y (cadr (gimp-drawable-offsets layer))) ;y 
			(area (* 1000 1000))
			(alpha (car (gimp-drawable-has-alpha layer)))
		    (sel (car (gimp-selection-is-empty image)))
		    (layer-name (car (gimp-drawable-get-name layer)))
			(layerList (vector->list (cadr (gimp-image-get-layers image))))
			(layerId 0)
		    (selection 0)
			(shadow-layer 0)
			(tsl-layer-mask 0)
			(plasma-layer 0)
			(bg-layer 0)
			(glow-layer 0)
			(size 150)
			(grow (/ size 30))
            (feather (/ size 4))
			(*newpoint* (cons-array 6 'double))
			(visible 0)
			(pasted 0)
			(offx (/ (- width layer-width) 2))
			(offy (/ (- height layer-height) 2))
			
        )
    
;;;;set all layers except the active layer to invisible		
	(while (not (null? layerList))
          (set! layerId (car layerList))
		  (gimp-drawable-set-visible layerId FALSE)
		  (set! layerList (cdr layerList)))
	(gimp-drawable-set-visible layer TRUE)
	
;;;;centre the layer        
    (gimp-layer-set-offsets layer offx offy)
;;;;scale the layer to the image size	
	(gimp-layer-scale layer (car (gimp-image-width image)) (car (gimp-image-height image)) TRUE)
	
	(gimp-context-push)
    (gimp-context-set-default-colors)

;;;;scale image to given area if required	
	(gimp-image-scale image 
		(max 1 (min 262144 (round (* width (sqrt (/ area (* width height)))))))
		(max 1 (min 262144 (round (* height (sqrt (/ area (* width height))))))))
	(set! width (car (gimp-image-width image)))
	(set! height (car (gimp-image-height image)))
	
	
	(if (= alpha FALSE) (gimp-layer-add-alpha layer))
	
;;;;check that a selection was made if not make one	
	(if (= sel TRUE) (gimp-selection-layer-alpha layer))
	
;;;;create selection (gimp-selection-load selection)    
	(set! selection (car (gimp-selection-save image)))	
	(gimp-drawable-set-name selection "selection")
    (gimp-image-set-active-layer image layer)	

;;;;begin the script
    (gimp-context-set-pattern pattern)
	(gimp-edit-fill layer PATTERN-FILL)
	
	(set! shadow-layer (car (gimp-layer-new image width height RGBA-IMAGE "Shadow" 100 MULTIPLY-MODE)))
    (gimp-image-add-layer image shadow-layer (car (gimp-image-get-layer-position image layer)))	
;;;;create the mask	
	(set! tsl-layer-mask (car (gimp-layer-create-mask shadow-layer ADD-BLACK-MASK)))
	(gimp-layer-add-mask shadow-layer tsl-layer-mask)	
    
	(gimp-context-set-background '(255 255 255))
    (gimp-edit-fill tsl-layer-mask BACKGROUND-FILL)
    (gimp-selection-feather image 15)

    (gimp-context-set-background '(0 0 0))
    (gimp-edit-fill shadow-layer BACKGROUND-FILL)
	(gimp-context-set-foreground '(255 255 255))

    (gimp-edit-blend shadow-layer FG-BG-RGB-MODE NORMAL-MODE
                     GRADIENT-SHAPEBURST-ANGULAR 100 0 REPEAT-NONE FALSE
                     FALSE 0 0 TRUE
                     0 0 1 1)
	
	(gimp-selection-none image)

	(set! plasma-layer (car (gimp-layer-new image width height RGBA-IMAGE "Plasma-layer" 100 NORMAL-MODE)))
    (gimp-image-add-layer image plasma-layer (car (gimp-image-get-layer-position image layer)))
    (gimp-context-set-background '(0 0 255))
	(gimp-drawable-fill plasma-layer BACKGROUND-FILL)
	(plug-in-solid-noise RUN-NONINTERACTIVE image plasma-layer 0 0 0 1 16 16)
	(gimp-threshold plasma-layer (+ 80 rust-coverage) 172)
	(plug-in-colortoalpha RUN-NONINTERACTIVE image plasma-layer '(255 255 255))
	(gimp-selection-layer-alpha plasma-layer)
    (script-fu-distress-selection image 
	                              plasma-layer 
								  127       ;Threshold (bigger 1<-->255 smaller)
								  8         ;Spread (8 0 1000 1 10 0 1)
								  4         ;Granularity (1 is low) (4 1 25 1 10 0 1)
								  2         ;Smooth (2 1 150 1 10 0 1)
								  TRUE      ;Smooth horizontally TRUE
								  TRUE)     ;Smooth vertically TRUE	
	
	(gimp-edit-clear plasma-layer)
	(gimp-selection-invert image)
	(gimp-context-set-foreground '(0 0 0))
	(gimp-edit-fill plasma-layer FOREGROUND-FILL)
	(gimp-selection-none image)
	
    (set! glow-layer (car (gimp-layer-new image width height RGBA-IMAGE "Glow" 100 NORMAL-MODE)))
	(gimp-image-add-layer image glow-layer (+ (car (gimp-image-get-layer-position image plasma-layer)) 1))
	
	(gimp-layer-set-lock-alpha plasma-layer TRUE)
	(gimp-context-set-background '(0 0 0))
    (gimp-edit-clear glow-layer)
    (gimp-selection-layer-alpha plasma-layer)
    (gimp-selection-grow image grow)
    (gimp-selection-feather image feather)
    (gimp-context-set-foreground '(255 143 43))
    (gimp-edit-fill glow-layer FOREGROUND-FILL)
    (gimp-selection-none image)
    (gimp-context-set-background '(0 0 0))
    (gimp-context-set-foreground '(79 79 79))
    (gimp-edit-blend plasma-layer FG-BG-RGB-MODE NORMAL-MODE GRADIENT-SHAPEBURST-ANGULAR 100 0 REPEAT-NONE FALSE FALSE 0 0 TRUE 0 0 1 1)
	(gimp-selection-load selection)
	(gimp-selection-invert image)
	(gimp-edit-clear glow-layer)
	(gimp-edit-clear plasma-layer)
	(gimp-selection-none image)
	
	                (aset *newpoint* 0 0)    ; set the arrays
					(aset *newpoint* 1 0)
					(aset *newpoint* 2 5)
	                (aset *newpoint* 3 metal-brightness)
					(aset *newpoint* 4 255)
					(aset *newpoint* 5 255)
	(gimp-curves-spline plasma-layer 0 6 *newpoint*)
	
	(gimp-selection-none image)
	(set! visible (car (gimp-layer-new image (car (gimp-image-width image)) (car (gimp-image-height image)) RGBA-IMAGE "Visible" 100 NORMAL-MODE)))
    (gimp-image-add-layer image visible 0)
	(gimp-edit-copy-visible image)
	(set! pasted (car (gimp-edit-paste visible FALSE)))
	(gimp-floating-sel-anchor pasted)
	(if (> severity 0) (plug-in-bump-map RUN-NONINTERACTIVE image visible plasma-layer 145 90 65 0 0 0 0 TRUE FALSE 0))
	(if (> bev-width 0) (the-rusted-beveler image visible
                               18 ;layer-mode
							   0  ;bev-type
							   0  ;bev-dir
							   bev-width
							   0  ;shape
							   0  ;contour
							   0  ;prenoise
							   135 ;light-angle
							   30 ;light-elevation
							   3 ;depth
							   3 ;postblur
							   10 ;gloss
							   FALSE ;keep-selection-in
							   TRUE)) ;merge
							   
	(set! visible (car (gimp-image-get-active-layer image)))
	(gimp-drawable-set-name visible "Visible")
	(if (> severity 0) (plug-in-bump-map RUN-NONINTERACTIVE image visible visible 135 45 severity 0 0 0 0 TRUE FALSE 0))
	
;;;;finish the script
	(if (= conserve FALSE) (begin
    (set! layer (car (gimp-image-merge-visible-layers image EXPAND-AS-NECESSARY)))
	(gimp-drawable-set-name layer layer-name)
    ))
;   (set! layer (car (gimp-image-get-active-layer image)))
    (gimp-image-remove-channel image selection)
	(if (and (= conserve FALSE) (= alpha FALSE)) (gimp-layer-flatten layer))
	(gimp-image-set-active-layer image layer)
;;;;Scale Image to it's original size;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (gimp-image-scale-full image final-width final-height 2)
	(if (= conserve FALSE) (begin
	(gimp-layer-scale layer layer-width layer-height TRUE)
	(gimp-layer-set-offsets layer x y)))
	
;;;;scale all visible layers
	(if (= conserve TRUE) (begin
    (set! layerList (vector->list (cadr (gimp-image-get-layers image))))
	(while (not (null? layerList))
    (set! layerId (car layerList))
	(if (= (car(gimp-layer-get-visible layerId)) TRUE) (begin
	(gimp-layer-scale layerId layer-width layer-height TRUE)
	(gimp-layer-set-offsets layerId x y)))
	(set! layerList (cdr layerList)))))

;;;;reset all layers to visible
    (set! layerList (vector->list (cadr (gimp-image-get-layers image))))
	(while (not (null? layerList))
    (set! layerId (car layerList))
	(gimp-drawable-set-visible layerId TRUE)
	(set! layerList (cdr layerList)))		  

	(gimp-displays-flush)
	(gimp-image-undo-group-end image)
	(gimp-context-pop)

 )
)

(script-fu-register "script-fu-rusted-alpha"        		    
  "Rusted..."
  "Can create a Rust like texture on a layer with alpha or within a selection"
  "Graechan"
  "Graechan - http://gimpchat.com"
  "Mar 2013"
  "RGB*"
  SF-IMAGE      "image"      0
  SF-DRAWABLE   "drawable"   0
  SF-PATTERN    "Rust Pattern"              "Fibers"
  SF-ADJUSTMENT "Rust Coverage" '(40 0 50 1 10 0 0)
  SF-ADJUSTMENT "Rust Severity" '(20 0 60 1 5 0 0)
  SF-ADJUSTMENT "Metal Brightness" '(100 80 255 1 10 0 0)
  SF-ADJUSTMENT "Bevel Width" '(20 1 100 1 10 0 0)
  SF-TOGGLE     "Keep the Layers"   FALSE
)

(script-fu-menu-register "script-fu-rusted-alpha" "<Image>/Filters/Alpha to Logo")



(define (script-fu-rusted-logo 
                                      img-width
                                      img-height
                                      text 
                                      color 
                                      font-in 
                                      font-size
                                      bkg-type 
                                      bkg-pattern
                                      bkg-color
									  gradient
									  pattern
									  rust-coverage
									  severity
							          metal-brightness
									  bev-width
							          conserve)
  (let* (
         (width img-width)
         (height img-height)
         (offx 0)
         (offy 0)
         (image (car (gimp-image-new img-width img-height RGB)))
         (area (* 1000 1000))
         (border (/ font-size 4))
		 (font (if (> (string-length font-in) 0) font-in (car (gimp-context-get-font))))
         (text-layer (car (gimp-text-fontname image -1 0 0 text border TRUE font-size PIXELS font)))
         (text-width (car (gimp-drawable-width text-layer)))
         (text-height (car (gimp-drawable-height text-layer)))
		 (final-width 0)
		 (final-height 0)
		 (bkg-layer 0)
		 (active-gradient (car (gimp-context-get-gradient)))
		 (active-fg (car (gimp-context-get-foreground)))
		 (active-bg (car (gimp-context-get-background)))
		 
         
         )
    (gimp-context-push)
	(gimp-context-set-foreground '(0 0 0))
	(gimp-context-set-background '(255 255 255))
	
;;;;centre text on line
	(gimp-text-layer-set-justification text-layer 2)
	
;;;;set the new Image size
	(if (> text-width img-width) (set! width text-width))           
    (if (> text-height img-height) (set! height text-height))
	(set! final-width width)
	(set! final-height height)	

;;;;resize the image	
	(gimp-image-resize image width height 0 0)

;;;;centre the text layer	
    (set! offx (/ (- width text-width) 2))
    (set! offy (/ (- height text-height) 2))    
    (gimp-layer-set-offsets text-layer offx offy)

;;;;set the text clolor    
    (gimp-context-set-foreground color)
	(gimp-selection-layer-alpha text-layer)
	(gimp-edit-fill text-layer FOREGROUND-FILL)
	(gimp-selection-none image)
;;;;start of script;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (script-fu-rusted-alpha image text-layer pattern rust-coverage severity metal-brightness bev-width conserve)
	
;;;;end of script;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
    
;;;;create the background layer    
	(if (> bkg-type 0) (begin
	(set! bkg-layer (car (gimp-layer-new image width height RGBA-IMAGE "Background" 100 NORMAL-MODE)))
    (gimp-image-add-layer image bkg-layer (+ (car (gimp-image-get-layer-position image (car (gimp-image-get-active-layer image)))) 1))))
	(gimp-context-set-pattern bkg-pattern)
	(gimp-context-set-background bkg-color)
	(gimp-context-set-gradient gradient)
	(if (= bkg-type 4) (begin
	(gimp-context-set-gradient active-gradient)
	(gimp-context-set-foreground active-fg)
	(gimp-context-set-background active-bg)))
	(if (= bkg-type 1) (gimp-drawable-fill bkg-layer PATTERN-FILL))		
    (if (= bkg-type 2) (gimp-drawable-fill bkg-layer BACKGROUND-FILL))	
    (if (= bkg-type 3) (gimp-edit-blend bkg-layer CUSTOM-MODE NORMAL-MODE  GRADIENT-LINEAR 100 0 REPEAT-NONE FALSE FALSE 3 0.2 TRUE 0 0 width height));; GRADIENT-LINEAR ; GRADIENT-CONICAL-SYMMETRIC
    (if (= bkg-type 4) (gimp-edit-blend bkg-layer CUSTOM-MODE NORMAL-MODE  GRADIENT-LINEAR 100 0 REPEAT-NONE FALSE FALSE 3 0.2 TRUE 0 0 width height))
    
	(if (and (= conserve FALSE) (> bkg-type 0)) (begin
	(set! text-layer (car (gimp-image-merge-visible-layers image EXPAND-AS-NECESSARY)))
	
	(gimp-drawable-set-name text-layer text)))

	(gimp-context-pop)

    (gimp-display-new image)
    )
  ) 
(script-fu-register "script-fu-rusted-logo"
  "Rusted..."
  "Create a logo with a rust like texture"
  "Graechan"
  "Graechan - http://gimpchat.com"
  "May 2013"
  ""
  SF-ADJUSTMENT "Image width (pixels)" '(250 1 5000 1 10 0 1)
  SF-ADJUSTMENT "Image height(pixels)" '(250 1 5000 1 10 0 1)
  SF-TEXT       "Text"    "RUSTED"
  SF-COLOR      "Text color"         "Black"
  SF-FONT       "Font"               "Arial Bold"
  SF-ADJUSTMENT "Font size (pixels)" '(250 6 500 1 1 0 1)
  SF-OPTION "Background Type" '("None" "Pattern" "Color" "Gradient" "Active Gradient")
  SF-PATTERN    "Background Pattern"            "Bricks"
  SF-COLOR      "Background color"         "White"
  SF-GRADIENT   "Background Gradient" "Abstract 3"
  SF-PATTERN    "Rust Pattern"              "Fibers"
  SF-ADJUSTMENT "Rust Coverage" '(40 0 50 1 10 0 0)
  SF-ADJUSTMENT "Rust Severity" '(20 0 60 1 5 0 0)
  SF-ADJUSTMENT "Metal Brightness" '(100 80 255 1 10 0 0)
  SF-ADJUSTMENT "Bevel Width" '(20 0 100 1 10 0 0)
  SF-TOGGLE     "Keep the Layers"   FALSE 
  )
(script-fu-menu-register "script-fu-rusted-logo" "<Image>/File/Create/Logos")

(define (the-rusted-beveler image layer
                               layer-mode
							   bev-type
							   bev-dir
							   bev-width
							   shape
							   contour
							   prenoise
							   light-angle
							   light-elevation
							   depth
							   postblur
							   gloss
							   keep-selection-in
							   merge
							   )
							  

 (let* (
            (width (car (gimp-image-width image)))
			(height (car (gimp-image-height image)))
			(alpha (car (gimp-drawable-has-alpha layer)))
			(layer-name (car (gimp-drawable-get-name layer)))
			(image-channel 0)
			(bevel-layer 0)
			(name 0)
			(*newpoint* (cons-array 6 'double))
		    (*newpointx1* (cons-array 16 'double))
			(y1 0)
			(y2 0)
			(y3 0)
			(y4 0)
			(y5 0)
			(y6 0)
			(keep-selection keep-selection-in)
			(selection-bounds 0)
			(bounds-x1 0)
			(bounds-y1 0)
			(bounds-x2 0)
			(bounds-y2 0)
						
        )
	
	
	(gimp-context-push)
    (gimp-image-undo-group-start image)
	(gimp-context-set-foreground '(0 0 0))
	(gimp-context-set-background '(255 255 255))

	(if (= alpha FALSE) (gimp-layer-add-alpha layer))
	
;;;;check that a selection was made if not make one	
;	(if (= (car (gimp-selection-is-empty image)) TRUE) (set! keep-selection FALSE))
	(if (= (car (gimp-selection-is-empty image)) TRUE) (gimp-selection-layer-alpha layer))

;;;;find the selection co-ordinates	
	(set! selection-bounds (cdr (gimp-selection-bounds image)))
	(set! bounds-x1 (car selection-bounds));x coordinate of upper left corner of selection bounds
    (set! bounds-y1 (cadr selection-bounds));y coordinate of upper left corner of selection bounds
    (set! bounds-x2 (caddr selection-bounds));x coordinate of lower right corner of selection bounds
    (set! bounds-y2 (cadddr selection-bounds));y coordinate of lower right corner of selection bounds	

;;;;save the selection    
	(set! image-channel (car (gimp-selection-save image)))	
	(gimp-drawable-set-name image-channel "image-channel")
	(gimp-image-set-active-layer image layer)
	
;;;;Add new layer with Bevel
    (if (= bev-type 0) (set! name "Inner bevel"))
    (if (= bev-type 1) (set! name "Outer bevel"))
    (gimp-selection-load image-channel)
    (gimp-image-set-active-layer image layer)    
    (set! bevel-layer (car (gimp-layer-new image width height RGBA-IMAGE name 100 layer-mode)))
    (gimp-image-add-layer image bevel-layer -1)
;;;;make outer bevel visible
	(if (= bev-type 1) (gimp-layer-set-mode bevel-layer NORMAL-MODE)) ;need to make outer bevel visible for scrpit to work
    (gimp-image-set-active-layer image bevel-layer)    
    
	(gimp-context-set-foreground '(0 0 0))
    (gimp-context-set-background '(255 255 255))
	(gimp-drawable-fill bevel-layer FOREGROUND-FILL)    
    (gimp-selection-grow image 3)
    (gimp-selection-feather image bev-width)
    (if (= bev-type 0) (gimp-selection-shrink image (/ bev-width 2)))
	(if (= bev-type 1) (gimp-selection-grow image (- (/ bev-width 2) (/ bev-width 5))))
    (gimp-edit-fill bevel-layer BACKGROUND-FILL)
;;;;apply shape curve
    (if (or (< shape 0) (> shape 0))
	(begin
					(aset *newpoint* 0 0)    ; set the arrays
					(aset *newpoint* 1 0)
					(aset *newpoint* 2 (round (+ (/ shape 10) 127)))
	                (aset *newpoint* 3 (+ shape 127))
					(aset *newpoint* 4 255)
					(aset *newpoint* 5 255)
	(gimp-curves-spline bevel-layer 0 6 *newpoint*)
   )
    )
;;;;apply the shape contours
    (if (> contour 0) (curves-contour bevel-layer 0 contour))	

    (gimp-selection-all image)
	
;;;;invert colours of whole layer if down                
    (if (= bev-dir 1) (gimp-invert bevel-layer))
;;;;add prenoise
    (if (> prenoise 0) (plug-in-hsv-noise 1 image bevel-layer 4 0 0 prenoise))
                        						
;;;;emboss the selection    
    (plug-in-emboss RUN-NONINTERACTIVE image bevel-layer light-angle light-elevation depth 1);Emboss

;;;;gloss the bevel-layer    
     (if (> gloss 0)
				(begin
                    (set! y1 (* 1.0 gloss))
					(set! y2 (* 3.0 gloss)) 
                    (set! y3 (* 9.6 gloss))
                    (set! y4 (* 3.2 gloss))
                    (set! y5 (* 4.0 gloss))
                    (set! y6 (* 0.4 gloss))
					
					(aset *newpointx1* 0 0)    ; set the arrays
					(aset *newpointx1* 1 0)
					(aset *newpointx1* 2 63)
	                (aset *newpointx1* 3 (+ 63 y1))
					(aset *newpointx1* 4 95)
					(aset *newpointx1* 5 (+ 95 y2))
					(aset *newpointx1* 6 127)
					(aset *newpointx1* 7 (- 127 y3))
					(aset *newpointx1* 8 156)
					(aset *newpointx1* 9 (+ 156 y4))
					(aset *newpointx1* 10 191)
					(aset *newpointx1* 11 (- 191 y5))
					(aset *newpointx1* 12 223)
					(aset *newpointx1* 13 (+ 223 y6))
					(aset *newpointx1* 14 255)
					(aset *newpointx1* 15 255)
					;(0,0, 63,(63 + y1), 95,(95 + y2), 127,(127 - y3), 156,(156 + y4), 191,(191 - y5), 223,(223 + y6), 255,255)
	 
	(gimp-curves-spline bevel-layer 0 16 *newpointx1*)
	 )
	 )
	
;;;;finish the layers
	(gimp-selection-load image-channel)
    (if (= bev-type 0) (begin
	(gimp-selection-shrink image (- bev-width (round (/ bev-width 3))))
    (gimp-edit-clear bevel-layer)))
  	(if (= bev-type 1) (begin
	(gimp-selection-grow image (+ bev-width 1))
	(gimp-selection-invert image)
	(gimp-edit-clear layer)
	(gimp-edit-clear bevel-layer)))
	
;postblur                
	(if (> postblur 0) (plug-in-gauss-rle2 RUN-NONINTERACTIVE image bevel-layer postblur postblur))
	
	(gimp-layer-set-lock-alpha bevel-layer FALSE)
	
;reload selection
    (gimp-selection-load image-channel)
                
;clear excess
    (if (= bev-type 0) 
	(begin
	(gimp-selection-shrink image 1)
	(gimp-selection-invert image) 
	(gimp-edit-clear bevel-layer)
	(gimp-edit-clear layer)
	(gimp-selection-invert image)))

	(if (= bev-type 1) 
	(begin
	(gimp-selection-shrink image 1)
	(gimp-edit-clear bevel-layer)))
    
 
;;;;Clean up the layers
    (if (= keep-selection FALSE) (gimp-selection-none image))
	(gimp-image-remove-channel image image-channel)
	(if (= alpha FALSE) (gimp-layer-flatten layer))
	(gimp-image-set-active-layer image bevel-layer)
	(if (= merge TRUE) (set! layer (car (gimp-image-merge-down image bevel-layer EXPAND-AS-NECESSARY))))
	(gimp-drawable-set-name layer layer-name)

	(gimp-displays-flush)
	(gimp-image-undo-group-end image)
	(gimp-context-pop)
    
 )
) 
